<?php


namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\GER\Services\CrudService;
use Doctrine\DBAL\Connection;
use Exception;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

use Teknisa\PLA\Util\RtfToHtml\Document;
use Teknisa\PLA\Util\RtfToHtml\HtmlFormatter;

class Pla08800HistoricoDaUnidadeController extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $zhFileUploader;
    protected $fileServerURL;
    protected $environment;
    protected $dataSourceName = 'pla08800_historico_da_unidade';
    
    public function __construct( $crudService, $parameterBag, $entityManager, $zhFileUploader, $fileServerURL, $environment) {
    	$this->crudService = $crudService;
    	$this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->zhFileUploader = $zhFileUploader;
        $this->fileServerURL = $fileServerURL;
        $this->environment = $environment;
    }

    function isRtf($str){
        return $str && !empty($str) && strpos($str, "rtf") !== false || true;
    }

    public function getObsHistFilial (DTO\Request\Filter $request, DTO\Response $response){
        
        parent::find($request, $response);
        $rows = $response->getDataSets()[0]->getRows();
        $response->setDataSets(array()); // Limpa os datasets para retornar apenas o dataSet convertido com utf8
        foreach($rows as &$row){
            if($row['DSOBSHISTFILIAL']){
                $row['DSOBSHISTFILIAL'] = utf8_encode(str_replace(chr(0), "",$row['DSOBSHISTFILIAL']));
                $row['ORIGINAL'] = $row['DSOBSHISTFILIAL'];
                $row['LENGTH'] = strlen($row['DSOBSHISTFILIAL']);
                 
                if($this->isRtf($row['DSOBSHISTFILIAL'])){
                    try{
                        $formatter = new HtmlFormatter('UTF-8');
                        $document = new Document($row['DSOBSHISTFILIAL']);
                        $row['DSOBSHISTFILIAL'] = $formatter->Format($document);
                    }catch(\Exception $e){
                    }
                 }
            }
        }
        $response->addDataSet(new DataSource\DataSet($this->dataSourceName, $rows));
        
    }
    
    private function salvaHistoricoUnidade($row, $operation){
        $sqlUpdate = "UPDATE OBSHISTFILIAL 
            SET DSOBSHISTFILIAL = V_RAW_TEMP, CDOPERULTATU = :P_CDOPERADOR
            WHERE CDFILIAL = :P_CDFILIAL 
                AND DTHISTFIL = :P_DTHISTFIL 
                AND CDDESCOBSFILIAL = :P_CDDESCOBSFILIAL;";

        $sqlInsert = "INSERT INTO OBSHISTFILIAL 
              (CDFILIAL,    DTHISTFIL,    CDDESCOBSFILIAL,    DSOBSHISTFILIAL, CDOPERADOR)  
        VALUES(:P_CDFILIAL, :P_DTHISTFIL, :P_CDDESCOBSFILIAL, V_RAW_TEMP,      :P_CDOPERADOR);";
        
        $sql = $operation == "INSERT" ? $sqlInsert : $sqlUpdate;
        $plSql = "
        DECLARE
            V_LENGTH NUMBER;
            V_RAW_TEMP LONG RAW;
        begin
            V_LENGTH := 32760;
            DBMS_LOB.READ(:P_DSOBSHISTFILIAL, V_LENGTH, 1, V_RAW_TEMP);
            $sql
        end;";
        $statement = $this->entityManager->getConnection()->prepare($plSql);
        $cdoperador = $this->environment->getCdOperador();
        $statement->bindParam(':P_CDFILIAL',        $row["CDFILIAL"],                       \PDO::PARAM_STR);
        $statement->bindParam(':P_DTHISTFIL',       $row["DTHISTFIL"],                      \PDO::PARAM_STR);
        $statement->bindParam(':P_CDDESCOBSFILIAL', $row["CDDESCOBSFILIAL"],                \PDO::PARAM_STR);
        $statement->bindParam(':P_DSOBSHISTFILIAL', $row["DSOBSHISTFILIAL"],                \PDO::PARAM_LOB);
        $statement->bindParam(':P_CDOPERADOR',      $cdoperador,    \PDO::PARAM_STR);
        $statement->execute();
    }

    public function saveObserv (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        
        $row["DSOBSHISTFILIAL"] = $row['RTF'];
        $operation = "";
        if (isset($row['__is_new']) && $row['__is_new']){
            $operation = "INSERT";
        } else {
            $operation = "UPDATE";
        }
        try {
            $connection = $this->entityManager->getConnection();;
            $connection->beginTransaction();
            
            $this->salvaHistoricoUnidade($row, $operation);
            $this->salvaAnexos($row);
            
            $connection->commit();
        } catch (Exception $e){
            $connection->rollback();
            throw $e;
        }
    }
    //Anexo
    private function setFileFullPath($dataSet) {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["DOCUMENTOANEXO"] !== null) {
				$row["DOCUMENTOANEXOFULLPATH"] = $this->fileServerURL.$row["DOCUMENTOANEXO"];
			}
		}
		return new DataSet("pla08800_historico_da_unidade_anexo", $rows);
	}

    public function anexosFind(DTO\Request $request, DTO\Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        $dataSet = $this->crudService->getFilteredDataset("pla08800_historico_da_unidade_anexo", $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
        $newDataSet = $this->setFileFullPath($dataSet);
        $response->addDataSet($newDataSet);
    }
    
    public function uploadAnexo(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->dataSourceName = "anx_obshistfilial";
            if (isset($row["fileUpload"]) && isset($row["fileUpload"][0]) && isset($row["fileUpload"][0]["name"]) && isset($row["fileUpload"][0]["path"])) {
                $row["DSDOCUMENTOANEXO"] = $row["fileUpload"][0]["name"];
                $row["DOCUMENTOANEXO"] = $row["fileUpload"][0]["path"];
            } else {
                throw new \Exception("errorSavingFile", 1);
            }
            $dataSet = new DataSet($this->dataSourceName, array($row));
            $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
            parent::save($newRequest, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteAnexo(DTO\Request $request, DTO\Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $this->dataSourceName = "pla08800_historico_da_unidade_anexo";
            foreach ($rows as $row) {
                $this->zhFileUploader->deleteFile($row["DOCUMENTOANEXO"]);
            }
            parent::delete($request, $response);
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }

    public function downloadAnexo(DTO\Request $request, DTO\Response $response) {
        $row = $request->getDataSet()->getRows()[0];
        $file = $row["DOCUMENTOANEXOFULLPATH"];
        $name = $row["DSDOCUMENTOANEXO"];
        $nameArray = explode(".", $name);
        $type = end($nameArray);
        header('Accept-Ranges: none');
        header("Content-Type: " . $type);
        header("Content-Disposition: attachment; filename=" . $name);
        readfile($file);
        exit;
    }
    
    private function novoCodigo($contador, $quantidade=1){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'NOVO_CODIGO_NT');
        $procedure->addParam(new Param('P_CONTADOR',   Param::PARAM_INPUT,  $contador,        Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEQUENCIAL', Param::PARAM_OUTPUT,  NULL,      Param::PARAM_TYPE_STR, 20));
        $procedure->addParam(new Param('P_QTDE',       Param::PARAM_INPUT,  $quantidade, Param::PARAM_TYPE_STR));
        $resultado = $procedure->execute();
        return $resultado['P_SEQUENCIAL'];
    }
    
    private function salvaAnexos($row){
        if(isset($row['fileUpload'])){
            if(is_array($row['fileUpload'])){
                $sql = "INSERT INTO ANX_OBSHISTFILIAL
                        (NRANEXOHIST, CDFILIAL, DTHISTFIL, CDOPERADOR, CDDESCOBSFILIAL,
                        DSDOCUMENTOANEXO, DOCUMENTOANEXO,
                        NRORG, NRORGINCLUSAO, NRORGULTATU,
                        CDOPERINCLUSAO, CDOPERULTATU,
                        DTINCLUSAO, DTULTATU)
                        VALUES
                        (:NRANEXOHIST, :CDFILIAL, :DTHISTFIL, :CDOPERADOR, :CDDESCOBSFILIAL,
                        :DSDOCUMENTOANEXO, :DOCUMENTOANEXO,
                        :NRORG, :NRORG, :NRORG,
                        :CDOPERADOR, :CDOPERADOR,
                        SYSDATE, SYSDATE)";
                $statement = $this->entityManager->getConnection()->prepare($sql);
                $cdoperador = $this->environment->getCdOperador();
                $nrorg = $this->environment->getNrOrgTrab();
                
                for($i = 0; $i < count($row['fileUpload']); $i++){
                    $params = [
                        'NRANEXOHIST' => $this->novoCodigo('NRANEXOHIST'),
                        'CDFILIAL' => $row['CDFILIAL'],
                        'DTHISTFIL' => $row['DTHISTFIL'],
                        'CDOPERADOR' => $cdoperador,
                        'CDDESCOBSFILIAL' => $row["CDDESCOBSFILIAL"],
                        'DSDOCUMENTOANEXO' => $row["fileUpload"][$i]["name"],
                        'DOCUMENTOANEXO' => $row["fileUpload"][$i]["path"],
                        'NRORG' => $nrorg,
                        'CDOPERADOR' => $cdoperador
                    ];
                    $statement->execute($params);
                }
            }
        }
    }
}